// EndPointController.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <stdio.h>
#include <wchar.h>
#include <tchar.h>
#include "windows.h"
#include "Mmdeviceapi.h"
#include "PolicyConfig.h"
#include "Propidl.h"
#include "Functiondiscoverykeys_devpkey.h"
#include <iostream>
#include <fstream>
#include <string> 

using namespace std;



HRESULT SetDefaultAudioPlaybackDevice(LPCWSTR devID)
{	
	IPolicyConfigVista *pPolicyConfig;
	ERole reserved = eConsole;

    HRESULT hr = CoCreateInstance(__uuidof(CPolicyConfigVistaClient), 
		NULL, CLSCTX_ALL, __uuidof(IPolicyConfigVista), (LPVOID *)&pPolicyConfig);
	if (SUCCEEDED(hr))
	{
		hr = pPolicyConfig->SetDefaultEndpoint(devID, reserved);
		pPolicyConfig->Release();
	}
	return hr;
}

bool cvtLPW2stdstring(std::string& s, const LPWSTR pw,
                      UINT codepage = CP_ACP)
{
    bool res = false;
    char* p = 0;
    int bsz;

    bsz = WideCharToMultiByte(codepage,
        0,
        pw,-1,
        0,0,
        0,0);
    if (bsz > 0) {
        p = new char[bsz];
        int rc = WideCharToMultiByte(codepage,
            0,
            pw,-1,
            p,bsz,
            0,0);
        if (rc != 0) {
            p[bsz-1] = 0;
            s = p;
            res = true;
        }
    }
    delete [] p;
    return res;
}

int _tmain(int argc, _TCHAR* argv[])
{
	// read the command line option
	int option = -1;
	if (argc == 2) 
	{
		option = atoi((char*)argv[1]);
		if(_ttoi(argv[1]) == -2) 
			option = -2;
	}

	HRESULT hr = CoInitialize(NULL);
	if (SUCCEEDED(hr))
	{
		IMMDeviceEnumerator *pEnum = NULL;
		// Create a multimedia device enumerator.
		hr = CoCreateInstance(__uuidof(MMDeviceEnumerator), NULL,
			CLSCTX_ALL, __uuidof(IMMDeviceEnumerator), (void**)&pEnum);
		if (SUCCEEDED(hr))
		{
			IMMDeviceCollection *pDevices;
			// get default audio device
			IMMDevice *defaultDevice = NULL;						
			PROPVARIANT friendlyDefault;
			PropVariantInit(&friendlyDefault);
			IPropertyStore *pStoreDefault;
			pEnum->GetDefaultAudioEndpoint(eRender, eConsole, &defaultDevice);
			defaultDevice->OpenPropertyStore(STGM_READ, &pStoreDefault);
			pStoreDefault->GetValue(PKEY_Device_DeviceDesc, &friendlyDefault);
			if (option == -1)
				printf("Default Audio Device: %ws\n", friendlyDefault.pwszVal);
			LPWSTR wstrIDDefault = NULL;
			defaultDevice->GetId(&wstrIDDefault);
			// Enumerate the output devices.
			hr = pEnum->EnumAudioEndpoints(eRender, DEVICE_STATE_ACTIVE, &pDevices);
			if (SUCCEEDED(hr))
			{
				UINT count;
				pDevices->GetCount(&count);
				if (SUCCEEDED(hr))
				{
					bool nextAudioToDefault = false;
					LPWSTR wstrFirstDeviceID = NULL;
					for (int i = 0; i < count; i++)
					{
						IMMDevice *pDevice;
						hr = pDevices->Item(i, &pDevice);
						if (SUCCEEDED(hr))
						{
							LPWSTR wstrID = NULL;
							hr = pDevice->GetId(&wstrID);
							if(i == 0)
								wstrFirstDeviceID = wstrID;
							if (SUCCEEDED(hr))
							{
								IPropertyStore *pStore;
								hr = pDevice->OpenPropertyStore(STGM_READ, &pStore);
								if (SUCCEEDED(hr))
								{
									PROPVARIANT friendlyName;
									PropVariantInit(&friendlyName);
									hr = pStore->GetValue(PKEY_Device_DeviceDesc, &friendlyName);
									if (SUCCEEDED(hr))
									{
										//if argument -2 set default audio device to the next device
										if(option == -2)
										{
											if(nextAudioToDefault)
											{
												SetDefaultAudioPlaybackDevice(wstrID);
												nextAudioToDefault = false;
											}
											string defaultID("\?"), currentID("\?");
											cvtLPW2stdstring(defaultID,wstrIDDefault);
											cvtLPW2stdstring(currentID,wstrID);
											if(defaultID == currentID)
												nextAudioToDefault = true;
										}

										// if no options, print the device
										// otherwise, find the selected device and set it to be default
										if (option == -1) printf("Audio Device %d: %ws\n",i, friendlyName.pwszVal);
										if (i == option) SetDefaultAudioPlaybackDevice(wstrID);
										PropVariantClear(&friendlyName);
									}

									pStore->Release();
								}
							}
							pDevice->Release();
						}
					}
					//if last device was default the next default device is first
					if(nextAudioToDefault)
						SetDefaultAudioPlaybackDevice(wstrFirstDeviceID);
				}
				pDevices->Release();
			}
			pEnum->Release();
		}
	}
	return hr;
}